﻿local L = AceLibrary("AceLocale-2.2"):new("ClassTimer")
local candy = AceLibrary("CandyBar-2.0")
local PaintChips = AceLibrary("PaintChips-2.0")
local surface = AceLibrary("Surface-1.0")

ClassTimer.defaults.bars = {
	growup = false,
	scale = 1,
	texture = "BantoBar",
	darken = true,
	Reversed = false,
	Size = 11,
	text = "%s (%a) (%t)",
	icon = false,
	colors = {}
}
ClassTimer.Menu.args[L["Bar Options"]] = {
	type = "group",
	name = L["Bar Options"],
	desc = L["Bar Options"],
	order = 11,
	args = { 
		[L["Show Anchor"]] = {
			type = "execute",
			order = 1,
			name = L["Show Anchor"],
			desc = L["Show the bar anchor frame."],
			func = function() ClassTimer:ShowAnchors() end,
		},
		[L["Visuals"]] = {
			type = "group",
			desc = L["Visual options for bars"],
			order = 2,
			name = L["Visuals"],
			args = {
				[L["Grow upwards"]] = {
					type = "toggle",
					order = 1,
					name = L["Grow upwards"],
					desc = L["Toggle bars grow upwards/downwards from anchor."],
					get = function() return ClassTimer.db.profile.growup end,
					set = function(u) ClassTimer.db.profile.growup = u ClassTimer:SetCandyBarGroupPoint("ClassTimerGroup", u and "BOTTOM" or "TOP", f, u and "TOP" or "BOTTOM", 0, 0)
	ClassTimer:SetCandyBarGroupGrowth("ClassTimerGroup", u) end,
				},
				[L["Only show icons"]] = {
					type = "toggle",
					name = L["Only show icons"],
					desc = L["Hide the bars and show only the icons and times"],
					order = 1,
					get = function() return ClassTimer.db.profile.icon end,
					set = function(v) ClassTimer.db.profile.icon = v end,
				},
				[L["Fill Bars"]] = {
					type = "toggle",
					order = 2,
					name = L["Fill Bars"],
					desc = L["Set the bars to fill."],
					get = function() return ClassTimer.db.profile.Reversed end,
					set = function() ClassTimer.db.profile.Reversed = v end,
				},
				[L["Text"]] = {
					type = "group",
					order = 3,
					name = L["Text"],
					desc = L["Options for text."],
					args = {
						[L["Text"]] = {
							type = "text",
							name = L["General text"],
							desc = L["%s for spell\n%t for target\n%a for applications"],
							get = function() return ClassTimer.db.profile.text end,
							set = function(v) ClassTimer.db.profile.class.text =  v end,
							usage = "",
						},						
						[L["Font Size"]] = {
							type = "range",
							name = L["Font Size"],
							desc = L["Set the bar font size."],
							min = 5,
							max = 20,
							step = 0.5,
							get = function() return ClassTimer.db.profile.Size end,
							set = function(v) ClassTimer.db.profile.Size = v end,
						},
					},
				},
				[L["Scale"]] = {
					type = "range",
					order = 4,
					name = L["Scale"],
					desc = L["Set the bar scale."],
					min = 0.2,
					max = 2.0,
					step = 0.1,
					get = function() return ClassTimer.db.profile.scale end,
					set = function(v) ClassTimer.db.profile.scale = v end,
				},
				[L["Texture"]] = {
					type = "text",
					order = 5,
					name = L["Texture"],
					desc = L["Set the texture for the timer bars."],
					get = function() return ClassTimer.db.profile.texture end,
					set = function(v) ClassTimer.db.profile.texture = v end,
					validate = surface:List(),
				},
				[L["Colors"]] = {
					type = "group",
					order = 6,
					name = L["Colors"],
					desc = L["Colors of messages and bars."],
					args = {
						[L["Background"]] = {
							name = L["Background"],
							type = "color",
							desc = L["Change the bar background color."],
							hasAlpha = true,
							get = function() if ClassTimer.db.profile.colors.bgc then local _, r, g, b = PaintChips:GetRGBPercent(ClassTimer.db.profile.colors.bgc); return r, g, b, ClassTimer.db.profile.colors.bga else return  0, .5, .5, .5 end end,
							set = function(r, g, b, a) local hex = ClassTimer:RGBToHex(r, g, b); PaintChips:RegisterHex(hex); ClassTimer.db.profile.colors.bgc = hex; ClassTimer.db.profile.colors.bga = a end,
						},
						[L["Text"]] = {
							name = L["Text"],
							type = "color",
							desc = L["Change the bar text color."],
							get = function() if ClassTimer.db.profile.colors.txtc then local _, r, g, b = PaintChips:GetRGBPercent(ClassTimer.db.profile.colors.txtc); return r, g, b else return  1, 1, 1 end end,
							set = function(r, g, b, a) local hex = ClassTimer:RGBToHex(r, g, b); PaintChips:RegisterHex(hex); ClassTimer.db.profile.colors.txtc = hex end,
						},
						[L["Darken"]] = {
							name = L["Darken"],
							desc = L["Darken the bars as the time left decreases"],
							order = 1,
							type = 'toggle',
							get = function()
							        return ClassTimer.db.profile.darken
							    end,
							set = function(v)
								ClassTimer.db.profile.darken = v
							end,
							},
						[L["Main Color"]] = {
						name = L["Main Color"],
						desc = L["Main color of bar if static color is set"],
						order = 2,
						type = "color",
						get = function() if ClassTimer.db.profile.colors.mainc then local _, r, g, b = PaintChips:GetRGBPercent(ClassTimer.db.profile.colors.mainc); return r, g, b else return  1, 1, 0 end end,
						set = function(r, g, b, a) local hex = ClassTimer:RGBToHex(r, g, b); PaintChips:RegisterHex(hex); ClassTimer.db.profile.colors.mainc = hex end,
						disabled = function() return ClassTimer.db.profile.darken end, 
						}
					}
				}
			}
		}
	}
}

function ClassTimer:RegHex(hex)
	for _,hexx in pairs(hex) do
		PaintChips:RegisterHex(hexx)
	end
end

function ClassTimer:BarSetup()
	if not surface:Fetch(self.db.profile.texture) then self.db.profile.texture = "BantoBar" end
	self:SetupFrames()
	self:RegHex(self.db.profile.colors)
	self:RegisterCandyBarGroup("ClassTimerGroup")
	local u = ClassTimer.db.profile.growup 
	ClassTimer:SetCandyBarGroupPoint("ClassTimerGroup", u and "BOTTOM" or "TOP", self.frames, u and "TOP" or "BOTTOM", 0, 0)
	ClassTimer:SetCandyBarGroupGrowth("ClassTimerGroup", u)
end

function ClassTimer:RGBToHex(r, g, b)
	return format("%02x%02x%02x", r*255, g*255, b*255)
end

function ClassTimer:ShowAnchors()
	self.frames:Show()
end

local string_gsub = string.gsub
function ClassTimer:GetText(spell, target, applications)
	local string = self.db.profile.text
	string = string_gsub(string, "%%s", spell)
	if applications and applications > 1 then
		string = string_gsub(string, "%%a", applications)
	else
		string = string_gsub(string, "%%a", "")
	end
	if target then
		string =string_gsub(string, "%%t", target)
	else
		string = string_gsub(string, "%%t", "")
	end
	string = string_gsub(string, "%(%)", "")
	string = string_gsub(string, "  ", " ")
	return string
end

function ClassTimer:StartBar(id, spell, target, applications, time, timeleft, icon)
	if self.db.profile.icon then
		ClassTimer:StartIcon(id, spell, target, applications, time, timeleft, icon)
		return
	end
	
	if not spell or not time then return end
	
	local text = self:GetText(spell, target, applications)
	
	if self.db.profile.darken then 
		self:RegisterCandyBar(id, time, text, icon, "Green", "Red")
	else
		self:RegisterCandyBar(id, time, text, icon, self.db.profile.colors.mainc)
	end
	self:SetCandyBarFontSize(id, self.db.profile.Size)
	self:SetCandyBarReversed(id, self.db.profile.Reversed)
	self:RegisterCandyBarWithGroup(id, "ClassTimerGroup")
	self:SetCandyBarTexture( id, surface:Fetch( self.db.profile.texture) )
	
	bc, balpha, txtc = self.db.profile.colors.bgc, self.db.profile.colors.bga, self.db.profile.colors.txtc
	if bc then self:SetCandyBarBackgroundColor(id, bc, balpha) end
	if txtc then self:SetCandyBarTextColor(id, txtc) end

	self:SetCandyBarScale(id, self.db.profile.scale)
	self:SetCandyBarFade(id, .5)
	
	self:SetCandyBarOnClick(id, function(id, button) self:OnClick(id, button) end)

	self:StartCandyBar(id, true)
	if timeleft then
		self:SetCandyBarTimeLeft(id, timeleft)
	end
end

function ClassTimer:StartIcon(id, spell, target, applications, time, timeleft, icon)
	if not time then return end
	self:RegisterCandyBar(id, time, "", icon)
	self:RegisterCandyBarWithGroup(id, "ClassTimerGroup")
	
	local spark = candy.handlers[id].frame.spark
	spark:Hide()
	
	txtc = self.db.profile.colors.txtc
	if txtc then self:SetCandyBarTextColor(id, txtc) end

	self:SetCandyBarWidth(id, 1)
	self:SetCandyBarScale(id, self.db.profile.scale)
	self:SetCandyBarFade(id, .5)
	self:StartCandyBar(id, true)
	self:SetCandyBarOnClick(id, function(id, button) self:OnCandyClick(id, button) end)
	if timeleft then
		self:SetCandyBarTimeLeft(id, timeleft)
	end
end

local function time(t)
	local timetext
	local m = floor(t/60)
	local s = t - (m*60)
	if m > 0 then
		timetext = string.format("%d:%02d mins", m, floor(s))
	elseif s < 10 then
		timetext = string.format("%1.1f secs", s)
	else
		timetext = string.format("%.0f secs", floor(s))
	end
return timetext
end

function ClassTimer:OnCandyClick(id, button)
	local msg = candy.handlers[id].text.." has "..time(candy.handlers[id].time-candy.handlers[id].elapsed).." remaining"
	if button == "LeftButton" then
		if UnitInRaid("player") then
			SendChatMessage(msg, "RAID")
		elseif UnitInParty("player") then
			SendChatMessage(msg, "PARTY")
		end
	elseif button == "RightButton" then
		self:UnregisterCandyBar(id)
	end
end

function ClassTimer:Test()
	self:StartBar("ClassTimer_Test1", L["Testing..."], nil, nil, 15, nil, self.hasIcon)
	self:StartBar("ClassTimer_Test2", L["Wait! a Test!"], nil, nil, 5, nil, self.hasIcon)
end 

function ClassTimer:SetupFrames()
	local f = GameFontNormal:GetFont()

	local frame = CreateFrame("Frame", nil, UIParent)
	self.frames = frame
	frame:Hide()

	frame:SetWidth(175)
	frame:SetHeight(75)
	frame:SetBackdrop({
		bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	})
	frame:SetBackdropBorderColor(.5, .5, .5)
	frame:SetBackdropColor(0,0,0)
	frame:ClearAllPoints()
	frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetMovable(true)
	frame:SetScript("OnDragStart", function() this:StartMoving() end)
	frame:SetScript("OnDragStop", function() this:StopMovingOrSizing() self:SavePosition() end)

	local header = frame:CreateFontString(nil,"OVERLAY")
	self.frames.header = header
	header:SetFont(f, 14)
	header:SetWidth(150)
	header:SetText("ClassTimer Bars")
	header:SetTextColor(1, .8, 0)
	header:ClearAllPoints()
	header:SetPoint("TOP", frame, "TOP", 0, -10)

	local leftbutton = CreateFrame("Button", nil, frame)
	self.frames.leftbutton = leftbutton
	leftbutton:SetWidth(40)
	leftbutton:SetHeight(25)
	leftbutton:SetPoint("RIGHT", frame, "CENTER", -10, -15)
	leftbutton:SetScript( "OnClick", function() self:Test() end )

	local lbt = leftbutton:CreateTexture()
	self.frames.leftbutton.texture = lbt
	lbt:SetWidth(50)
	lbt:SetHeight(32)
	lbt:SetPoint("CENTER", leftbutton, "CENTER")
	lbt:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
	lbt:SetTexCoord(0, 0.625, 0, 0.6875)
	leftbutton:SetNormalTexture(lbt)

	local lbtext = leftbutton:CreateFontString(nil,"OVERLAY")
	self.frames.leftbutton.text = lbtext
	lbtext:SetFontObject(GameFontHighlight)
	lbtext:SetText("Test")
	lbtext:SetAllPoints(leftbutton)

	local rightbutton = CreateFrame("Button", nil, frame)
	self.frames.rightbutton = rightbutton
	rightbutton:SetWidth(40)
	rightbutton:SetHeight(25)
	rightbutton:SetPoint("LEFT", frame, "CENTER", 10, -15)
	rightbutton:SetScript( "OnClick", function() frame:Hide() end )

	local rbt = rightbutton:CreateTexture()
	self.frames.rightbutton.texture = rbt
	rbt:SetWidth(50)
	rbt:SetHeight(32)
	rbt:SetPoint("CENTER", rightbutton, "CENTER")
	rbt:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
	rbt:SetTexCoord(0, 0.625, 0, 0.6875)
	rightbutton:SetNormalTexture(rbt)

	local rbtext = rightbutton:CreateFontString(nil,"OVERLAY")
	self.frames.rightbutton.text = rbtext
	rbtext:SetFontObject(GameFontHighlight)
	rbtext:SetText("Close")
	rbtext:SetAllPoints(rightbutton)

	self:RestorePosition()
end

function ClassTimer:SavePosition()
	local f = self.frames
	local s = f:GetEffectiveScale()

	self.db.profile.posx = f:GetLeft() * s
	self.db.profile.posy = f:GetTop() * s
end

function ClassTimer:RestorePosition()
	local x = self.db.profile.posx
	local y = self.db.profile.posy

	if not x or not y then return end

	local f = self.frames
	local s = f:GetEffectiveScale()

	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
end 